// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =					 All rights reserved.
// ====================================================================================
// 
//  __	    __	    __
//  \ \	   /  \	   / /   [File name   ] video_top.v
//   \ \  / /\ \  / /	 [Description ] Video demo
//	  \ \/ /  \ \/ /	 [Timestamp   ] Friday May 26 14:00:30 2019
//	   \  /	   \  /	     [version	  ] 1.0.0
//	    \/	    \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:	|  Author	| Mod. Date	| Changes Made:
// ----------------------------------------------------------------------------------
// V1.0	| Caojie	 | 11/22/19	 | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========



module video_top
(
	input             I_clk           , //50Mhz 
	input             I_key1          ,
	output     [3:0]  O_led           ,  
	inout	          IO_adv7513_scl  ,
	inout	          IO_adv7513_sda  ,
	output            O_adv7513_clk    ,
	output            O_adv7513_vs    ,
	output            O_adv7513_hs    ,
	output            O_adv7513_de    ,
	output     [23:0] O_adv7513_data     
);

//==================================================
wire        rst_n1;
wire        rst_n2;

//--------------------------
reg  [31:0] run_cnt;
wire        running;

//--------------------------
wire        pixel_clk;

//--------------------------
wire        tp0_vs_in  ;
wire        tp0_hs_in  ;
wire        tp0_de_in ;
wire [ 7:0] tp0_data_r;
wire [ 7:0] tp0_data_g;
wire [ 7:0] tp0_data_b;

reg         vs_r;
reg  [9:0]  cnt_vs;

//---------------------------
wire [7:0] I_din0;       
wire [7:0] I_din1;       
wire [7:0] I_din2;       

 
wire        de_yc ;
wire [7:0]  data_y ;
wire [7:0]  data_cb ;
wire [7:0]  data_cr ;


wire        de_rgb ;  
wire [7:0]  data_rgb_r ;  
wire [7:0]  data_rgb_g ;  
wire [7:0]  data_rgb_b ;  

reg  [11:0]  vs_dn;
reg  [11:0]  hs_dn;

//------------------------------------------
//ADV7513 I2c
wire         TX_EN_7513;
wire [2:0]   WADDR_7513;
wire [7:0]   WDATA_7513;
wire         RX_EN_7513;
wire [2:0]   RADDR_7513;
wire [7:0]   RDATA_7513;

wire         sclk;
// wire         clk_osc;

// OSC osc_inst 
// (
    // .OSCOUT(clk_osc) //250/4=62.5MHz 
// );
// defparam osc_inst.FREQ_DIV = 4;

Gowin_rPLL Gowin_rPLL_inst
(
    .clkout (pixel_clk), // 
    .clkoutp(sclk     ), //50MHz
    .clkin  (I_clk    )  
);

//==================================================
key_debounceN #
(
	.DEBCNT1 (32'd50000000 ),//1s
	.DEBCNT2 (32'd100000000) //2s
) 
key_debounceN_inst0
(
	.clk       (sclk   ), 
	.rst_n     (1'b1    ), 
	.key_in    (I_key1  ),//I_rst_n ), 
	.key_n_out1(rst_n1  ),
	.key_n_out2(rst_n2  )
);


//===================================================
//LED灯测试
always @(posedge sclk or negedge rst_n1)
begin
	if(!rst_n1)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd50_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;


assign  O_led[0] = 1'b0;
assign  O_led[1] = running;
assign  O_led[2] = rst_n1;
assign  O_led[3] = rst_n2;
 


testpattern testpattern_inst
(
    .I_pxl_clk   (pixel_clk          ),//pixel clock
    .I_rst_n     (rst_n1             ),//low active 
    .I_mode      ({1'b0,cnt_vs[8:7]} ),//data select
    .I_sqr_width (16'd60             ),
    .I_single_r  (8'd0               ),
    .I_single_g  (8'd255             ),
    .I_single_b  (8'd0               ),                  //1280x720      //1920x1080(148.5MHz)
    .I_h_total   (16'd1650           ),//hor total time  // 16'd1650     // 16'd2200
    .I_h_sync    (16'd40             ),//hor sync time   // 16'd40       // 16'd44  
    .I_h_bporch  (16'd220            ),//hor back porch  // 16'd220      // 16'd148 
    .I_h_res     (16'd1280           ),//hor resolution  // 16'd1280     // 16'd1920
    .I_v_total   (16'd750            ),//ver total time  // 16'd750      // 16'd1125
    .I_v_sync    (16'd5              ),//ver sync time   // 16'd5        // 16'd5   
    .I_v_bporch  (16'd20             ),//ver back porch  // 16'd20       // 16'd36  
    .I_v_res     (16'd720            ),//ver resolution  // 16'd720      // 16'd1080
    .I_hs_pol    (1'b1               ),//HS polarity , 0:negetive ploarity，1：positive polarity
    .I_vs_pol    (1'b1               ),//VS polarity , 0:negetive ploarity，1：positive polarity
    .O_de        (tp0_de_in          ),   
    .O_hs        (tp0_hs_in          ),
    .O_vs        (tp0_vs_in          ),
    .O_data_r    (tp0_data_r         ),   
    .O_data_g    (tp0_data_g         ),
    .O_data_b    (tp0_data_b         )
);

always@(posedge pixel_clk)//clkout )
begin
	vs_r<=tp0_vs_in;
end

always@(posedge pixel_clk or negedge rst_n1)//clkout )
begin
	if(!rst_n1)
		cnt_vs<=10'd0;
	else if(vs_r && !tp0_vs_in) //vs24 falling edge
		cnt_vs<=cnt_vs+1'b1;
end 

//================================================================

assign I_din0 = tp0_data_r;
assign I_din1 = tp0_data_g;
assign I_din2 = tp0_data_b;

//-------------------------------------
//rgb to yc444
rgb_yc_top rgb_yc_top_inst
(
    .I_rst_n     (rst_n1    ), //reset signal, low active                     
	.I_clk       (pixel_clk  ),
	.I_din0      (I_din0     ),
	.I_din1      (I_din1     ),
	.I_din2      (I_din2     ),
	.I_dinvalid  (tp0_de_in  ),
	.O_dout0     (data_y     ),
	.O_dout1     (data_cb    ),
	.O_dout2     (data_cr    ),
	.O_doutvalid (de_yc      )
);

//-------------------------------------
//yc444 to rgb
yc_rgb_top yc_rgb_top_inst
(
    .I_rst_n     (rst_n1    ), //reset signal, low active                     
	.I_clk       (pixel_clk  ),
	.I_din0      (data_y     ),
	.I_din1      (data_cb    ),
	.I_din2      (data_cr    ),
	.I_dinvalid  (de_yc      ),
	.O_dout0     (data_rgb_r ),
	.O_dout1     (data_rgb_g ),
	.O_dout2     (data_rgb_b ),
	.O_doutvalid (de_rgb     )
);

//-----------------------------------------
always @(posedge pixel_clk or negedge rst_n1)
begin
	if(!rst_n1)
		begin
			vs_dn <= 12'd0;
			hs_dn <= 12'd0;
		end
	else    
		begin                              
			vs_dn <= {vs_dn[10:0],tp0_vs_in};
			hs_dn <= {hs_dn[10:0],tp0_hs_in};
		end
end

////---------------------------------------------     
assign O_adv7513_data 	 = {data_rgb_r,data_rgb_g,data_rgb_b};//{r,g,b}
assign O_adv7513_vs      = vs_dn[11];//syn_off0_vs;
assign O_adv7513_hs      = hs_dn[11];//syn_off0_hs;
assign O_adv7513_de 	 = de_rgb;//off0_syn_de;
assign O_adv7513_clk     = pixel_clk;
 
 
//=========================================================================
// ADV7513 I2c 初始化
  adv7513_iic_init 	    adv7513_iic_init_inst0
  (
      .I_CLK              ( sclk                  ),
      .I_RESETN           ( rst_n1                ),//low active
	  .start              ( rst_n2                ),
      .O_TX_EN            ( TX_EN_7513            ),
      .O_WADDR            ( WADDR_7513            ),
      .O_WDATA            ( WDATA_7513            ),
      .O_RX_EN            ( RX_EN_7513            ),
      .O_RADDR            ( RADDR_7513            ),
      .I_RDATA            ( RDATA_7513            ),
      .cstate_flag        (                       ),
      .error_flag         (                       )
  );



  I2C_MASTER_Top        I2C_MASTER_Top_inst0
  (
      .I_CLK              ( sclk                 ),
      .I_RESETN           ( rst_n2                 ),//low active
      .I_TX_EN            ( TX_EN_7513            ),
      .I_WADDR            ( WADDR_7513            ),
      .I_WDATA            ( WDATA_7513            ),
      .I_RX_EN            ( RX_EN_7513            ),
      .I_RADDR            ( RADDR_7513            ),
      .O_RDATA            ( RDATA_7513            ),
      .O_IIC_INT          (                       ),
      .SCL                ( IO_adv7513_scl           ),
      .SDA                ( IO_adv7513_sda           )
  );

endmodule