//---------------------------------------------------------------------
// File name  : tb_top.v
// Module name: tb_top
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date       | AUTHOR     |    DESCRIPTION
// ---------------------------------------------------------------------
//   1.0   | 24-Sep-2009 | Caojie     |    initial
// ---------------------------------------------------------------------

`timescale 1ns / 1ps

module tb();

//========================================================
//parameters
parameter BMP_VIDEO_FORMAT        = "WxH_xHz"; //video format
parameter BMP_PIXEL_CLK_FREQ      = 150.000;//pixel clock frequency, unit: MHz
parameter BMP_PIXEL_CLK_PERIOD    = 1000.0/BMP_PIXEL_CLK_FREQ; //unit: ns
parameter BMP_WIDTH               = 320;
parameter BMP_HEIGHT              = 240;
parameter BMP_OPENED_NAME         = "../../tb/pic/img320.bmp";
parameter BMP_REPEAT              = 1'b1;  //0:bmp increase  , 1:bmp repeat 
parameter BMP_LINK                = 1'b0;  //0:أ1:˫

parameter BMP_OUTPUTED_WIDTH      = BMP_WIDTH;
parameter BMP_OUTPUTED_HEIGHT     = BMP_HEIGHT;
parameter BMP_OUTPUTED_NAME       = "../../tb/pic/out001.bmp";
parameter BMP_OUTPUTED_NUMBER     = 16'd2;


//=======================================================
reg  pixel_clock;

reg  rst_n;
reg  video_gen_en;

//-------------
//driver
wire       vsync; 
wire       hsync; 
wire       data_valid; 
wire [7:0] data0_r; 
wire [7:0] data0_g;
wire [7:0] data0_b;
wire [7:0] data1_r ; 
wire [7:0] data1_g ;
wire [7:0] data1_b ;

wire [7:0] I_din0;
wire [7:0] I_din1;
wire [7:0] I_din2;

 
wire        de_yc;
wire [7:0]  data_y;
wire [7:0]  data_cb;
wire [7:0]  data_cr;


wire        de_rgb;  
wire [7:0]  data_rgb_r;
wire [7:0]  data_rgb_g;
wire [7:0]  data_rgb_b;


//-----------------
//monitor rgb input
wire        m_clk;
wire        m_vs_rgb;  
wire        m_hs_rgb;  
wire        m_de_rgb;  
wire [7:0]  m_data0_r;
wire [7:0]  m_data0_g;
wire [7:0]  m_data0_b;
wire [7:0]  m_data1_r;
wire [7:0]  m_data1_g;
wire [7:0]  m_data1_b;

//=====================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

//=====================================================
//pixel_clock
initial
  begin
    pixel_clock = 1'b0;
  end

always  #(BMP_PIXEL_CLK_PERIOD/2.0) pixel_clock = ~pixel_clock;

//=====================================================
//rst_n
initial
  begin
    rst_n=1'b0;
    
    #2000;
    rst_n=1'b1;
end

initial
  begin
    video_gen_en=1'b0;
    
    #4000;
    video_gen_en=1'b1;
end

//==================================================
//video driver
driver #
(
    .BMP_VIDEO_FORMAT    (BMP_VIDEO_FORMAT   ),
    .BMP_PIXEL_CLK_FREQ  (BMP_PIXEL_CLK_FREQ ),
    .BMP_WIDTH           (BMP_WIDTH          ),
    .BMP_HEIGHT          (BMP_HEIGHT         ),
    .BMP_OPENED_NAME     (BMP_OPENED_NAME    )
)
driver_inst
(
    .link_i      (BMP_LINK    ), //0,أ1˫
    .repeat_en   (BMP_REPEAT  ),
    .video_gen_en(video_gen_en),
    .pixel_clock (pixel_clock ),
    .vsync       (vsync       ), 
    .hsync       (hsync       ), 
    .data_valid  (data_valid  ),
    .data0_r     (data0_r     ), 
    .data0_g     (data0_g     ),
    .data0_b     (data0_b     ), 
    .data1_r     (data1_r     ), 
    .data1_g     (data1_g     ),
    .data1_b     (data1_b     )
);

//==============================================
//dut
	assign I_din0 = data0_r ;
	assign I_din1 = data0_g ;
	assign I_din2 = data0_b ;


//-------------------------------------
//rgb to yc444
rgb_yc_top rgb_yc_top_inst
(
    .I_rst_n     (rst_n      ), //reset signal, low active                     
	.I_clk       (pixel_clock),
	.I_din0      (I_din0     ),
	.I_din1      (I_din1     ),
	.I_din2      (I_din2     ),
	.I_dinvalid  (data_valid ),
	.O_dout0     (data_y     ),
	.O_dout1     (data_cb    ),
	.O_dout2     (data_cr    ),
	.O_doutvalid (de_yc      )
);

//-------------------------------------
//yc444 to rgb
yc_rgb_top yc_rgb_top_inst
(
    .I_rst_n     (rst_n      ), //reset signal, low active                     
	.I_clk       (pixel_clock),
	.I_din0      (data_y     ),
	.I_din1      (data_cb    ),
	.I_din2      (data_cr    ),
	.I_dinvalid  (de_yc      ),
	.O_dout0     (data_rgb_r ),
	.O_dout1     (data_rgb_g ),
	.O_dout2     (data_rgb_b ),
	.O_doutvalid (de_rgb     )
);

//==========================================================
//monitor
assign m_clk     = pixel_clock  	;
assign m_vs_rgb  = vsync        	;//vs_rgb         ;
assign m_hs_rgb  = hsync        	;//hs_rgb         ;
assign m_de_rgb  = de_rgb       	;
assign m_data0_r = data_rgb_r[7:0]	; //[9:2]  //[11:4]  //[7:0] 
assign m_data0_g = data_rgb_g[7:0]	; //[9:2]  //[11:4]  //[7:0]
assign m_data0_b = data_rgb_b[7:0]	; //[9:2]  //[11:4]  //[7:0]
assign m_data1_r = 8'd0         	;
assign m_data1_g = 8'd0         	;
assign m_data1_b = 8'd0         	;

monitor #
(
    .BMP_OUTPUTED_WIDTH  (BMP_OUTPUTED_WIDTH ),
    .BMP_OUTPUTED_HEIGHT (BMP_OUTPUTED_HEIGHT),
    .BMP_OUTPUTED_NAME   (BMP_OUTPUTED_NAME  ),
    .BMP_OUTPUTED_NUMBER (BMP_OUTPUTED_NUMBER)
)
monitor_inst
(
    .link_i       (BMP_LINK          ), //0,أ1˫
    .video2bmp_en (rst_n             ),
    .pixel_clock  (m_clk             ), 
    .vsync        (m_vs_rgb          ),        
    .hsync        (m_hs_rgb          ),        
    .data_valid   (m_de_rgb          ), 
    .data0_r      (m_data0_r         ),    
    .data0_g      (m_data0_g         ),
    .data0_b      (m_data0_b         ),
    .data1_r      (m_data1_r         ),    
    .data1_g      (m_data1_g         ),
    .data1_b      (m_data1_b         )
);
          
endmodule
